@@ -234,6 +234,13 @@ enum Command {
         namespace: String,
         #[structopt(flatten)]
         azure: AzureCredentials,
+        #[structopt(
+            help = "Kubeconfig context to use for updating cluster",
+            long,
+            short,
+            visible_alias = "ctx"
+        )]
+        context: Option<String>,
     },
 
     #[structopt(
@@ -440,6 +447,8 @@ enum Command {
     SetupConfigmap {
         #[structopt(help = "cluster name")]
         cluster: v0::ClusterName,
+        #[structopt(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
+        context: Option<String>,
         #[structopt(
             help = "Namespace to install statehub components",
             default_value = "statehub-system"
@@ -501,21 +510,30 @@ enum Command {
         display_order(1000),
         setting(clap::AppSettings::Hidden)
     )]
-    ListNamespaces,
+    ListNamespaces {
+        #[structopt(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
+        context: Option<String>,
+    },
 
     #[structopt(
         about = "List K8s nodes",
         display_order(1000),
         setting(clap::AppSettings::Hidden)
     )]
-    ListNodes,
+    ListNodes {
+        #[structopt(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
+        context: Option<String>,
+    },
 
     #[structopt(
         about = "List K8s pods",
         display_order(1000),
         setting(clap::AppSettings::Hidden)
     )]
-    ListPods,
+    ListPods {
+        #[structopt(help = "Kubeconfig context to use", long, short, visible_alias = "ctx")]
+        context: Option<String>,
+    },
 
     #[structopt(
         about = "List K8s node regions",
@@ -660,16 +678,16 @@ impl Cli {
                 name,
                 namespace,
                 azure,
+                context,
             } => {
-                let name = name.or_else(k8s::get_current_cluster_name).ok_or_else(|| {
-                    anyhow::anyhow!(
-                        "No default Kubernetes context found, need to provide cluster name"
-                    )
-                })?;
-                let kubectl = k8s::Kubectl::new()?;
-
+                // let name = name.or_else(k8s::get_current_cluster_name).ok_or_else(|| {
+                //     anyhow::anyhow!(
+                //         "No default Kubernetes context found, need to provide cluster name"
+                //     )
+                // })?;
+                // let kubectl = k8s::Kubectl::with_context_or_current(None)?;
                 statehub
-                    .update_cluster(kubectl, name, namespace, azure)
+                    .update_cluster(name, context, namespace, azure)
                     .await
             }
             Command::ListClusters => statehub.list_clusters().await,
@@ -742,13 +760,14 @@ impl Cli {
                 context,
             } => statehub.save_cluster_token(namespace, token, context).await,
             Command::SetupConfigmap {
-                namespace,
                 cluster,
+                context,
+                namespace,
                 default_state,
                 cleanup_grace,
             } => {
                 statehub
-                    .setup_configmap(namespace, cluster, default_state, cleanup_grace)
+                    .setup_configmap(cluster, context, namespace, default_state, cleanup_grace)
                     .await
             }
             Command::InstallClusterCrd {
@@ -767,9 +786,9 @@ impl Cli {
                 let kubectls = k8s::Kubectl::with_names_and_contexts(names, contexts)?;
                 statehub.uninstall_cluster_crd(kubectls).await
             }
-            Command::ListNamespaces => statehub.list_namespaces().await,
-            Command::ListNodes => statehub.list_nodes().await,
-            Command::ListPods => statehub.list_pods().await,
+            Command::ListNamespaces { context } => statehub.list_namespaces(context).await,
+            Command::ListNodes { context } => statehub.list_nodes(context).await,
+            Command::ListPods { context } => statehub.list_pods(context).await,
             Command::ListRegions { context, zone } => statehub.list_regions(context, zone).await,
             Command::SaveConfig => statehub.save_config().await,
             Command::SelfUpdate => statehub.self_update().await,
@@ -941,20 +960,27 @@ impl StateHub {
 
     async fn update_cluster(
         &self,
-        kubectl: k8s::Kubectl,
-        name: v0::ClusterName,
+        name: Option<v0::ClusterName>,
+        context: Option<String>,
         namespace: String,
         azure: AzureCredentials,
     ) -> anyhow::Result<()> {
-        self.inform(format_args!(
-            "Updating cluster with K8s context {} with {:?}",
-            name, azure
-        ))?;
+        let names = name.into_iter().collect();
+        let contexts = context.into_iter().collect();
+        let kubectls = k8s::Kubectl::with_names_and_contexts(names, contexts)?;
 
-        let azure = Some(&azure);
-        let aws = None;
-        self.setup_credentials_helper(&kubectl, &namespace, azure, aws)
-            .await
+        for kubectl in kubectls {
+            self.inform(format_args!(
+                "Updating cluster with K8s context {} with {:?}",
+                kubectl.cluster_name(),
+                azure
+            ))?;
+            let azure = Some(&azure);
+            let aws = None;
+            self.setup_credentials_helper(&kubectl, &namespace, azure, aws)
+                .await?;
+        }
+        Ok(())
     }
 
     async fn show_cluster(&self, name: v0::ClusterName) -> anyhow::Result<()> {
@@ -1207,22 +1233,28 @@ impl StateHub {
 
     async fn setup_configmap(
         &self,
+        cluster: v0::ClusterName,
+        context: Option<String>,
         namespace: String,
-        cluster_name: v0::ClusterName,
         default_state: Option<String>,
         cleanup_grace: String,
     ) -> anyhow::Result<()> {
+        let names = vec![cluster];
+        let contexts = context.into_iter().collect();
+        let kubectls = k8s::Kubectl::with_names_and_contexts(names, contexts)?;
         let default_state = default_state.as_deref().unwrap_or("");
         let api = self.api.url("");
-        k8s::Kubectl::with_context(&cluster_name)?
-            .store_configmap(
-                &namespace,
-                &cluster_name,
-                default_state,
-                &api,
-                &cleanup_grace,
-            )
-            .await?;
+        for kubectl in kubectls {
+            kubectl
+                .store_configmap(
+                    &namespace,
+                    kubectl.cluster_name(),
+                    default_state,
+                    &api,
+                    &cleanup_grace,
+                )
+                .await?;
+        }
 
         Ok(())
     }
@@ -1260,24 +1292,27 @@ impl StateHub {
             .print(&self.stdout, self.json)
     }
 
-    async fn list_namespaces(&self) -> anyhow::Result<()> {
-        k8s::list_namespaces()
+    async fn list_namespaces(&self, context: Option<String>) -> anyhow::Result<()> {
+        k8s::Kubectl::with_context_or_current(context.as_deref())?
+            .list_namespaces()
             .await?
             .into_iter()
             .for_each(|namespace| println!("{:#?}", namespace));
         Ok(())
     }
 
-    async fn list_nodes(&self) -> anyhow::Result<()> {
-        k8s::list_nodes()
+    async fn list_nodes(&self, context: Option<String>) -> anyhow::Result<()> {
+        k8s::Kubectl::with_context_or_current(context.as_deref())?
+            .list_nodes()
             .await?
             .into_iter()
             .for_each(|node| println!("{}", node.show()));
         Ok(())
     }
 
-    async fn list_pods(&self) -> anyhow::Result<()> {
-        k8s::list_pods()
+    async fn list_pods(&self, context: Option<String>) -> anyhow::Result<()> {
+        k8s::Kubectl::with_context_or_current(context.as_deref())?
+            .list_pods()
             .await?
             .into_iter()
             .for_each(|pod| println!("{}", pod.show()));
