@@ -93,7 +93,7 @@ impl StateHub {
             .install_statehub_cluster(namespace, register_cluster.install_state_crd)
             .await?;
 
-        self.setup_cluster_token_helper(&cluster, register_cluster)
+        self.setup_cluster_token_helper(&kubectl, &cluster, register_cluster)
             .await?;
         self.setup_credentials_helper(
             &kubectl,
@@ -331,12 +331,15 @@ or run following commands before running 'statehub register-cluster`
 
     pub(super) async fn setup_cluster_token_helper(
         &self,
+        kubectl: &k8s::Kubectl,
         cluster: &v0::Cluster,
         register_cluster: &RegisterCluster,
     ) -> anyhow::Result<()> {
         let token = self.api.issue_cluster_token(&cluster.name).await?;
         self.verbosely(format!("Issued token {} for {}", token.token, cluster))?;
-        k8s::store_cluster_token(&register_cluster.namespace, &token.token).await?;
+        kubectl
+            .store_cluster_token(&register_cluster.namespace, &token.token)
+            .await?;
         Ok(())
     }
 
